/* 
 * File:   Animation.cpp
 * Author: Trixt0r
 * 
 * Created on 19. April 2012, 20:40
 */
#include <map>
#include <list>
#include <algorithm>

#include "Animation.h"

using namespace std;

Animation::Animation(unsigned int bones, unsigned int frames) {
    this->x,y,angle,scale,parent_angle,parent_length,
            sprite_xscale,sprite_yscale,sprite_index,
            joint_xscale,joint_yscale,joint_index = new map<unsigned int,vector<double>* >();
    this->keyframes = new vector<bool>();
    this->bones = bones;
    for(int i = 0; i < bones; i++){
        x->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        y->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        angle->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        scale->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        parent_angle->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        parent_length->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        sprite_xscale->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        sprite_yscale->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        sprite_index->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        joint_xscale->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        joint_yscale->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        joint_index->insert(pair<unsigned int,vector<double>* >(i,new vector<double>()));
        /*for(int j = 0; j < frames; j++){
            x->at(i)->at(j) = 0;
            y->at(i)->at(j) = 0;
            angle->at(i)->at(j) = 0;
            scale->at(i)->at(j) = 0;
            parent_angle->at(i)->at(j) = 0;
            parent_length->at(i)->at(j) = 0;
            sprite_xscale->at(i)->at(j) = 0;
            sprite_yscale->at(i)->at(j) = 0;
            sprite_index->at(i)->at(j) = 0;
            joint_xscale->at(i)->at(j) = 0;
            joint_yscale->at(i)->at(j) = 0;
            joint_index->at(i)->at(j) = 0;
        }*/
    }
}

Animation::Animation(const Animation& orig) {
}

Animation::~Animation() {
    delete x,y,angle,scale,parent_angle,parent_length,keyframes,sprite_xscale,
            sprite_yscale,sprite_index,joint_xscale,joint_yscale,joint_index;
}

void Animation::add_frame(unsigned int index,double x, double y, double angle, double scale, double parent_angle, double parent_length,
            double sprite_xscale, double sprite_yscale, double sprite_index, double joint_xscale, double joint_yscale, double joint_index){
    this->keyframes->at(index) = false;
    this->x->at(index)->push_back(x);
    this->y->at(index)->push_back(y);
    this->angle->at(index)->push_back(angle);
    this->scale->at(index)->push_back(scale);
    this->parent_angle->at(index)->push_back(parent_angle);
    this->parent_length->at(index)->push_back(parent_length);
    this->sprite_xscale->at(index)->push_back(sprite_xscale);
    this->sprite_yscale->at(index)->push_back(sprite_yscale);
    this->sprite_index->at(index)->push_back(sprite_index);
    this->joint_xscale->at(index)->push_back(joint_xscale);
    this->joint_yscale->at(index)->push_back(joint_yscale);
    this->joint_index->at(index)->push_back(joint_index);
}

void Animation::set_keyframe(unsigned int index, const bool keyframe){
    this->keyframes->at(index) = keyframe;
}

int Animation::get_frames(){
    return this->keyframes->size();
}

int Animation::get_bones(){
    return this->bones;
}

void Animation::concat(Animation* animation){
    try{
        for(int i = 0; i < animation->get_bones(); i++){
            for(int j  = 0; j <  animation->get_frames(); j++){
                int frames = this->get_frames();
                this->keyframes->at(frames+i) = animation->keyframes->at(j);
                this->x->at(i)->at(frames+j) = animation->x->at(i)->at(j);
                this->y->at(i)->at(frames+j) = animation->y->at(i)->at(j);
                this->angle->at(i)->at(frames+j) = animation->angle->at(i)->at(j);
                this->scale->at(i)->at(frames+j) = animation->scale->at(i)->at(j);
                this->parent_angle->at(i)->at(frames+j) = animation->parent_angle->at(i)->at(j);
                this->parent_length->at(i)->at(frames+j) = animation->parent_length->at(i)->at(j);
                this->sprite_xscale->at(i)->at(frames+j) = animation->sprite_xscale->at(i)->at(j);
                this->sprite_yscale->at(i)->at(frames+j) = animation->sprite_yscale->at(i)->at(j);
                this->sprite_index->at(i)->at(frames+j) = animation->sprite_index->at(i)->at(j);
                this->joint_xscale->at(i)->at(frames+j) = animation->joint_xscale->at(i)->at(j);
                this->joint_yscale->at(i)->at(frames+j) = animation->joint_yscale->at(i)->at(j);
                this->joint_index->at(i)->at(frames+j) = animation->joint_index->at(i)->at(j);
            }
        }
    }
    catch(int e){
        return;
    }
}

Animation *Animation::create_between(Animation* animation, unsigned int frames){
    Animation *animation_new;
    animation_new = new Animation(this->get_bones(),this->get_frames());
    
    return animation_new;
}

void Animation::calc_between(unsigned int index){
   for(int i = 0; i < this->get_bones(); i++){
        int prev = this->find_prev_keyframe(index);//Find previous keyframe
        int between = i-prev;//Calc frames between keyframes
        //Calc differences
        double x_diff = (this->x->at(i)->at(index)-this->x->at(i)->at(prev))/between;
        double y_diff = (this->y->at(i)->at(index)-this->y->at(i)->at(prev))/between;
        double angle_diff = (this->angle->at(i)->at(index)-this->angle->at(i)->at(prev))/between;
        double scale_diff = (this->scale->at(i)->at(index)-this->scale->at(i)->at(prev))/between;
        double parent_angle_diff = (this->parent_angle->at(i)->at(index)-this->parent_angle->at(i)->at(prev))/between;
        double parent_length_diff = (this->parent_length->at(i)->at(index)-this->parent_length->at(i)->at(prev))/between;
        double sprite_xscale_diff = (this->sprite_xscale->at(i)->at(index)-this->sprite_xscale->at(i)->at(prev))/between;
        double sprite_yscale_diff = (this->sprite_yscale->at(i)->at(index)-this->sprite_yscale->at(i)->at(prev))/between;
        double sprite_index_diff = (this->sprite_index->at(i)->at(index)-this->sprite_index->at(i)->at(prev))/between;
        double joint_xscale_diff = (this->joint_xscale->at(i)->at(index)-this->joint_xscale->at(i)->at(prev))/between;
        double joint_yscale_diff = (this->joint_yscale->at(i)->at(index)-this->joint_yscale->at(i)->at(prev))/between;
        double joint_index_diff = (this->joint_index->at(i)->at(index)-this->joint_index->at(i)->at(prev))/between;
        //Calc non-keyframe data
        for(int j = prev+1; j < index; j++){
            this->x->at(i)->at(j) = x->at(i)->at(prev)+x_diff*(j-prev);
            this->y->at(i)->at(j) = y->at(i)->at(prev)+y_diff*(j-prev);
            this->angle->at(i)->at(j) = angle->at(i)->at(prev)+angle_diff*(j-prev);
            this->scale->at(i)->at(j) = angle->at(i)->at(prev)+scale_diff*(j-prev);
            this->parent_angle->at(i)->at(j) = parent_angle->at(i)->at(prev)+parent_angle_diff*(j-prev);
            this->parent_length->at(i)->at(j) = parent_length->at(i)->at(prev)+parent_length_diff*(j-prev);
            this->sprite_xscale->at(i)->at(j) = sprite_xscale->at(i)->at(prev)+sprite_xscale_diff*(j-prev);
            this->sprite_yscale->at(i)->at(j) = sprite_yscale->at(i)->at(prev)+sprite_yscale_diff*(j-prev);
            this->sprite_index->at(i)->at(j) = sprite_index->at(i)->at(prev)+sprite_index_diff*(j-prev);
            this->joint_xscale->at(i)->at(j) = joint_xscale->at(i)->at(prev)+joint_xscale_diff*(j-prev);
            this->joint_yscale->at(i)->at(j) = joint_yscale->at(i)->at(prev)+joint_yscale_diff*(j-prev);
            this->joint_index->at(i)->at(j) = joint_index->at(i)->at(prev)+joint_index_diff*(j-prev);
        }
        this->calc_between(this->find_next_keyframe(index));
    }
}

unsigned int Animation::find_prev_keyframe(int index){
    bool found = false;
    int i;
    for(i = index-1; i >= 0 && !found; --i)
        found = this->keyframes->at(i);
    return i+1;
}

unsigned int Animation::find_next_keyframe(int index){
    bool found = false;
    int i;
    for(i = index+1; i < this->get_frames() && !found; i++)
        found = this->keyframes->at(i);
    return i-1;
}