/* 
 * File:   Animation.h
 * Author: Trixt0r
 *
 * Created on 19. April 2012, 20:40
 */

#ifndef ANIMATION_H
#define	ANIMATION_H

#include "Skeleton.h"
#include <vector>
#include <map>

class Animation {
public:
    /*Creates a new frame in the animation and saves it at the end of the map.*/
    void add_frame(unsigned int index,double x, double y, double angle, double scale, double parent_angle, double parent_length,
            double sprite_xscale, double sprite_yscale, double sprite_index, double joint_xscale, double joint_yscale, double joint_index);
    /*Changes the given frame to a keyframe or not.*/
    void set_keyframe(unsigned int index, const bool keyframe);
    /*Returns the number of frames, the animation has.*/
    int get_frames();
    /*Returns the number of bones, the animation is applied to.*/
    int get_bones();
    /*Creates an animation between this one and the given and returns it.*/
    Animation *create_between(Animation *animation, unsigned int frames);
    /*Calculates the non-keyframes in the animation*/
    void calc_between(unsigned int index);
    /*Concats the animation with the given animation. New animation will be saved in the applied object.*/
    void concat(Animation *animation);
    Animation(unsigned int bones, unsigned int frames);
    Animation(const Animation& orig);
    virtual ~Animation();
    
    //Fields
    std::map<unsigned int,std::vector<double>* > *x,*y,*angle,*scale,*parent_angle,*parent_length,
            *sprite_xscale,*sprite_yscale,*sprite_index,
            *joint_xscale,*joint_yscale,*joint_index;
    std::vector<bool> *keyframes;
    int bones;
private:
    unsigned int find_prev_keyframe(int index);
    unsigned int find_next_keyframe(int index);
};

#endif	/* ANIMATION_H */

