/* 
 * File:   Bone.cpp
 * Author: Trixt0r
 * 
 * Created on 19. April 2012, 20:39
 */

#include "Bone.h"

using namespace std;

Bone::Bone(char *name, Bone *parent,double x,double y,double angle, double scale,unsigned int index) {
    this->children = new vector<Bone*>();
    this->name = name;
    this->parent = parent;
    this->x = x;
    this->y = y;
    this->angle = angle;
    this->scale = scale;
    this->xrot = 0;
    this->xscale = 1;
    this->yrot = 0;
    this->yscale = 1;
    this->origin_x = 0;
    this->origin_y = 0;
    this->skel_x = 0;
    this->skel_y = 0;
    this->anim_x = x;
    this->anim_y = y;
    this->added_angle = 0;
    this->animation = 0L;
    this->sprite = noone;
    this->joint = noone;
    this->index = index;
    if(this->parent != 0L) this->set_parent_data();
}

Bone::Bone(const Bone& orig) {
}

Bone::~Bone() {
    delete this->children, this->animation;
}

void Bone::set_sprite(double sprite_index,double x, double y, double depth, bool joint){
    double a,l;
    a = point_direction(this->x,this->y,this->x+x,this->y+y);
    l = point_distance(this->x,this->y,this->x+x,this->y+y);
    if(!joint){
        this->sprite = sprite_index;
        this->sprite_dir = a;
        this->sprite_length = l;
        this->sprite_depth = depth;
    }
    else{
        this->joint = sprite_index;
        this->joint_dir = a;
        this->joint_length = l;
        this->joint_depth = depth;
    }
}

void Bone::update(unsigned int frame){
    if(this->anim_exists()){
        anim_x = this->animation->x->at(index)->at(frame);
        anim_y = this->animation->y->at(index)->at(frame);
        angle = this->animation->angle->at(index)->at(frame);
        scale = this->animation->scale->at(index)->at(frame);
        parent_angle = this->animation->parent_angle->at(index)->at(frame);
        parent_length = this->animation->parent_length->at(index)->at(frame);
        sprite_xscale = this->animation->sprite_xscale->at(index)->at(frame)*xscale*yscale;
        sprite_yscale = this->animation->sprite_yscale->at(index)->at(frame);
        sprite_index = this->animation->sprite_index->at(index)->at(frame);
        joint_xscale = this->animation->joint_xscale->at(index)->at(frame)*xscale*yscale;
        joint_yscale = this->animation->joint_yscale->at(index)->at(frame);
        joint_index = this->animation->joint_index->at(index)->at(frame);
    }
    if(this->parent_exists()){
        x = parent->x+lengthdir_x(parent_length*(parent_scale/parent->scale)*xscale,parent_angle+parent->angle);
        y = parent->y+lengthdir_y(parent_length*(parent_scale/parent->scale)*yscale,parent_angle+parent->angle);
    }
    else{
        x = skel_x+(anim_x-origin_x)*xscale;
        y = skel_y+(anim_y-origin_y)*yscale;
    }
    if(yscale == 1)    angle = fmod(360+(xrot+xscale*(angle+added_angle)),360);
    else{
        if(xscale == 1)    angle = 180-fmod(angle+added_angle+180,360);
        else    angle = fmod(yrot*yscale+angle+added_angle,360);
    }
    this->update_sprites();
}

void Bone::update_sprites(){
    if(sprite != noone){
        double a = sprite_dir+angle*xscale;
        if(yscale >= 0){
            sprite_x = x+lengthdir_x(sprite_length,a*xscale);
            sprite_y = y+lengthdir_y(sprite_length,a*xscale);
        }
        else{
            if(xscale >= 0){
                a = sprite_dir+angle*yscale;
                sprite_x = x+lengthdir_x(sprite_length,-a);
                sprite_y = y+lengthdir_y(sprite_length,-a);
            }
            else{
                a = sprite_dir+angle;
                sprite_x = x+lengthdir_x(sprite_length,-a);
                sprite_y = y+lengthdir_y(xscale*sprite_length,-a);
            }
        }
    }
    if(joint != noone){
        double a = joint_dir+angle*xscale;
        if(yscale >= 0){
            joint_x = x+lengthdir_x(joint_length,a*xscale);
            joint_y = y+lengthdir_y(joint_length,a*xscale);
        }
        else{
            if(xscale >= 0){
                a = joint_dir+angle*yscale;
                joint_x = x+lengthdir_x(joint_length,-a);
                joint_y = y+lengthdir_y(joint_length,-a);
            }
            else{
                a = joint_dir+angle;
                joint_x = x+lengthdir_x(joint_length,-a);
                joint_y = y+lengthdir_y(xscale*joint_length,-a);
            }
        }
    }
}

bool Bone::anim_exists(){
    return this->animation != 0L;
}

bool Bone::parent_exists(){
    return this->parent != 0L;
}

void Bone::set_parent_data(){
    parent_scale = this->parent->scale;
    parent_angle = point_direction(this->parent->x,this->parent->y,this->x,this->y)-this->parent->angle;
    parent_length = point_distance(this->x,this->y,this->parent->x,this->parent->y)*this->parent->scale;
    this->parent->children->push_back(this);
}

double angle_difference(double angle_a, double angle_b){
    return fmod(fmod(angle_a - angle_b, 360) + 540, 360) - 180;
}

double lengthdir_x(double length, double angle){
    return cos(DEGTORAD(-angle))*length;
}
double lengthdir_y(double length, double angle){
    return sin(DEGTORAD(-angle))*length;
}

double point_direction(double x1, double y1, double x2, double y2){
    double x = x1-x2;
    double y = y1-y2;
    return atan2(-y,x)*(180/M_PI);
}

double point_distance(double x1, double y1, double x2, double y2){
    double x = x1-x2;
    double y = y1-y2;
    return sqrt((x*x)+(y*y))*(180/M_PI);
}